package com.me.brickbuster.entity;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.me.brickbuster.BrickBuster;
import com.me.brickbuster.Utils;
import net.dermetfan.utils.Pair;

public class Ball extends Entity {

	public static final int RADIUS = 18;
	public static final Color BALL_COLOR = Color.CHARTREUSE;

	public Vector2 direction;
	public float speed;

	public Ball(BrickBuster brickBuster) {
		super(brickBuster,BrickBuster.WIDTH/2, Paddle.PADDLE_Y + Paddle.PADDLE_HEIGHT + RADIUS);
	}

	@Override
	public void render() {
		getShapeRenderer().begin(ShapeType.Filled);
		getShapeRenderer().setColor(BALL_COLOR);
		getShapeRenderer().circle(getX(), getY(), RADIUS);
		getShapeRenderer().end();
	}

	@Override
	public void update(float dt) {
		if (!getBrickBuster().isPlaying()) {
			return;
		}

		float newX = getX() + (direction.x * speed);
		float newY = getY() + (direction.y * speed);

		if (newX + RADIUS > BrickBuster.WIDTH || newX - RADIUS < 0) {
			Utils.reflect(direction, BrickBuster.VERTICAL_EDGE);
		} else if (newY + RADIUS > BrickBuster.HEIGHT) {
			Utils.reflect(direction, BrickBuster.HORIZONTAL_EDGE);
		} else if (newY - RADIUS < 0) {
			reset();
		}

		if (direction.y < 0) {
			Pair<Vector2, Vector2> paddle = getBrickBuster().getPaddle().getTopEdge();
			Vector2 ball = new Vector2(newX, newY);
			Vector2 lineDir = paddle.getValue().sub(paddle.getKey());
			Vector2 nearest = Utils.nearestPoint(paddle.getKey().cpy(), lineDir, ball.cpy());

			if (nearest.dst(newX, newY) < RADIUS) {
				paddleCollision();
			}
		}

		setX(getX() + (direction.x * speed));
		setY(getY() + (direction.y * speed));
	}

	public void launch() {
		float angle = MathUtils.random(MathUtils.PI/2) + MathUtils.PI/4;
		direction = new Vector2(MathUtils.cos(angle), MathUtils.sin(angle));
		speed = 5;
	}

	public void paddleCollision() {
		float paddleCenter = getBrickBuster().getPaddle().getX() + Paddle.PADDLE_WIDTH/2;
		float rel = (getX() - paddleCenter) + 50;
		float newAngle = MathUtils.PI - (MathUtils.PI * (rel/100));
		direction = new Vector2(MathUtils.cos(newAngle), MathUtils.sin(newAngle));
	}

	public void reset() {
		setX(getBrickBuster().getPaddle().getX() + Paddle.PADDLE_WIDTH/2);
		setY(Paddle.PADDLE_Y + Paddle.PADDLE_HEIGHT + RADIUS);
		getBrickBuster().setPlaying(false);
	}
}
